@@ -22,6 +22,75 @@
 #include "sygpio_server.h"
 #include "sysyslog.h"
 
+SyGpioBundleEvent::SyGpioBundleEvent(SyGpioBundleEvent::Type type,
+				     const QHostAddress &orig_addr,
+				     uint16_t orig_port,
+				     int srcnum,const QString &code)
+{
+  event_type=type;
+  event_origin_address=orig_addr;
+  event_origin_port=orig_port;
+  event_source_number=srcnum;
+  event_code=code;
+}
+
+
+SyGpioBundleEvent::Type SyGpioBundleEvent::type() const
+{
+  return event_type;
+}
+
+
+QHostAddress SyGpioBundleEvent::originAddress() const
+{
+  return event_origin_address;
+}
+
+
+uint16_t SyGpioBundleEvent::originPort() const
+{
+  return event_origin_port;
+}
+
+
+QHostAddress SyGpioBundleEvent::sourceAddress() const
+{
+  return SyRouting::streamAddress(SyRouting::Stereo,event_source_number);
+}
+
+
+int SyGpioBundleEvent::sourceNumber() const
+{
+  return event_source_number;
+}
+
+
+QString SyGpioBundleEvent::code() const
+{
+  return event_code;
+}
+
+
+QString SyGpioBundleEvent::dump() const
+{
+  QString str="GPIO Bundle Event\n";
+  if(type()==SyGpioBundleEvent::TypeGpi) {
+    str+="Type: GPI\n";
+  }
+  else {
+    str+="Type: GPO\n";
+  }
+  str+=QString().sprintf("Source Number: %d\n",sourceNumber());
+  str+="Code: "+code()+"\n";
+  str+=QString().sprintf("Origin: %s:%d\n",
+			 (const char *)originAddress().toString().toUtf8(),
+			 originPort());
+  return str;
+}
+
+
+
+
 SyGpioEvent::SyGpioEvent(SyGpioEvent::Type type,const QHostAddress &orig_addr,
 			 uint16_t orig_port,int srcnum,int line,bool state,
 			 bool pulse)
@@ -54,6 +123,12 @@ uint16_t SyGpioEvent::originPort() const
 }
 
 
+QHostAddress SyGpioEvent::sourceAddress() const
+{
+  return SyRouting::streamAddress(SyRouting::Stereo,event_source_number);
+}
+
+
 int SyGpioEvent::sourceNumber() const
 {
   return event_source_number;
@@ -167,7 +242,7 @@ SyGpioServer::~SyGpioServer()
 }
 
 
-void SyGpioServer::sendGpi(int gpi,int line,bool state,bool pulse)
+void SyGpioServer::sendGpi(int srcnum,int line,bool state,bool pulse)
 {
   uint8_t data[28]={0x03,0x00,0x02,0x07,0xC6,0x04,0x55,0x1E,
 		    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
@@ -185,8 +260,8 @@ void SyGpioServer::sendGpi(int gpi,int line,bool state,bool pulse)
   //
   // Livewire Number
   //
-  data[23]=0xFF&(gpi>>8);
-  data[24]=0xFF&gpi;
+  data[23]=0xFF&(srcnum>>8);
+  data[24]=0xFF&srcnum;
 
   //
   // Line Number
@@ -216,7 +291,80 @@ void SyGpioServer::sendGpi(int gpi,int line,bool state,bool pulse)
 }
 
 
-void SyGpioServer::sendGpo(int gpo,int line,bool state,bool pulse)
+void SyGpioServer::sendGpi(int srcnum,const QString &code)
+{
+  int count=0;
+  /*
+  uint8_t data[28]={0x03,0x00,0x02,0x07,0xC6,0x04,0x55,0x1E,
+		    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+		    'I','N','D','I',0x00,0x01,
+		    0x00,0x00,0x00,0x00,0x07,0x00};
+  */
+  uint8_t data[52]={0x03,0x00,0x02,0x07,0xC6,0x04,0x55,0x1E,
+		    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+		    'I','N','D','I',0x00,0x01,
+		    0x00,0x00,0x00,0x00,0x07,0x00,
+		    0x00,0x00,0x00,0x00,0x07,0x00,
+		    0x00,0x00,0x00,0x00,0x07,0x00,
+		    0x00,0x00,0x00,0x00,0x07,0x00,
+		    0x00,0x00,0x00,0x00,0x07,0x00};
+
+  //
+  // Serial Number
+  //
+  data[4]=0xFF&(gpio_serial>>24);
+  data[5]=0xFF&(gpio_serial>>16);
+  data[6]=0xFF&(gpio_serial>>8);
+  data[7]=0xFF&gpio_serial;
+
+  //
+  // Build Events
+  //
+  count=0;
+  for(int i=0;i<SWITCHYARD_GPIO_BUNDLE_SIZE;i++) {
+    if((code.mid(i,1)=="H")||(code.mid(i,1)=="L")) {
+      unsigned offset=22+count*6;
+
+      //
+      // Livewire Number
+      //
+      data[offset+1]=0xFF&(srcnum>>8);
+      data[offset+2]=0xFF&srcnum;
+
+      //
+      // Line Number
+      //
+      data[offset+3]=0xFF&(0x0D-i);
+
+      //
+      // State
+      //
+      data[offset+5]=0xFF&(code.mid(i,1)=="L");
+
+      count++;
+    }
+  }
+  data[21]=0xFF&count;
+
+  //
+  // Send Packet
+  //
+  gpio_gpi_socket->writeDatagram((const char *)data,22+6*count,
+				 QHostAddress(SWITCHYARD_GPIO_ADDRESS),
+				 SWITCHYARD_GPIO_GPI_PORT);
+  gpio_serial++;
+  data[4]=0xFF&(gpio_serial>>24);
+  data[5]=0xFF&(gpio_serial>>16);
+  data[6]=0xFF&(gpio_serial>>8);
+  data[7]=0xFF&gpio_serial;
+  gpio_gpi_socket->writeDatagram((const char *)data,22+6*count,
+				 QHostAddress(SWITCHYARD_GPIO_ADDRESS),
+				 SWITCHYARD_GPIO_GPI_PORT);
+  gpio_serial+=2;
+}
+
+
+void SyGpioServer::sendGpo(int srcnum,int line,bool state,bool pulse)
 {
   uint8_t data[60]={0x03,0x00,0x02,0x07,0x36,0x0B,0x97,0xA9,
 		    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
@@ -238,8 +386,8 @@ void SyGpioServer::sendGpo(int gpo,int line,bool state,bool pulse)
   //
   // Livewire Number
   //
-  data[23]=0xFF&(gpo>>8);
-  data[24]=0xFF&gpo;
+  data[23]=0xFF&(srcnum>>8);
+  data[24]=0xFF&srcnum;
 
   //
   // Line Number
@@ -284,6 +432,11 @@ void SyGpioServer::sendGpo(int gpo,int line,bool state,bool pulse)
 }
 
 
+void SyGpioServer::sendGpo(int srcnum,const QString &code)
+{
+}
+
+
 void SyGpioServer::interfaceStartedData()
 {
   gpio_gpi_socket->subscribe(SWITCHYARD_GPIO_ADDRESS);
@@ -306,29 +459,51 @@ void SyGpioServer::gpiReadyReadData()
   QHostAddress addr;
   uint16_t port=0;
   SyGpioEvent *e=NULL;
+  SyGpioBundleEvent *eb=NULL;
+  int srcnum=0;
+  QString code;
 
   while((n=gpio_gpi_socket->readDatagram(data,1500,&addr,&port))>0) {
-    serial=((0xFF&data[4])<<24)+((0xFF&data[5])<<16)+((0xFF&data[6])<<8)+
-      (0xFF&data[7]);
-    if((gpio_src_addr_serials[addr.toIPv4Address()]!=serial)&&
-       (gpio_src_addr_serials[addr.toIPv4Address()]!=(serial-1))) {
-      gpio_src_addr_serials[addr.toIPv4Address()]=serial;
-      uint16_t count=((0xFF&data[20])<<8)+(0xFF&data[21]);
-      for(uint16_t i=0;i<count;i++) {
-	unsigned offset=22+i*6;
-	//
-	// FIXME: Is there actually a 'pulse' component for a GPI event?
-	//        We assume not here (even though there is a place for one
-	//        in the Switchyard API).
-	//
-	e=new SyGpioEvent(SyGpioEvent::TypeGpi,addr,port,
-			  ((0xFF&data[offset+1])<<8)+(0xFF&data[offset+2]),
-			  0x0D-(0xff&data[offset+3]),
-			  (data[offset+5]&0x01)!=0,false);
-	emit gpioReceived(e);
-	emit gpiReceived(e->sourceNumber(),e->line(),e->state(),e->isPulse());
-	gpio_routing->setGpi(e->sourceNumber(),e->line(),e->state(),e->isPulse());
-	delete e;
+    if(addr!=gpio_routing->nicAddress()) {
+      serial=((0xFF&data[4])<<24)+((0xFF&data[5])<<16)+((0xFF&data[6])<<8)+
+	(0xFF&data[7]);
+      if((gpio_src_addr_serials[addr.toIPv4Address()]!=serial)&&
+	 (gpio_src_addr_serials[addr.toIPv4Address()]!=(serial-1))) {
+	gpio_src_addr_serials[addr.toIPv4Address()]=serial;
+	uint16_t count=((0xFF&data[20])<<8)+(0xFF&data[21]);
+	srcnum=((0xFF&data[22+1])<<8)+(0xFF&data[22+2]);
+	code=gpio_gpi_codes.value(srcnum,"hhhhh");
+	for(uint16_t i=0;i<count;i++) {
+	  unsigned offset=22+i*6;
+	  //
+	  // FIXME: Is there actually a 'pulse' component for a GPI event?
+	  //        We assume not here (even though there is a place for one
+	  //        in the Switchyard API).
+	  //
+	  //	srcnum=((0xFF&data[offset+1])<<8)+(0xFF&data[offset+2]);
+	  int line=0x0D-(0xff&data[offset+3]);
+	  bool state=(data[offset+5]&0x01)!=0;
+	  //	printf("i: %d  srcnum: %d  line: %d  state: %d\n",
+	  //	       i,srcnum,line,state);
+	  if(state) {
+	    code.replace(line,1,"L");
+	  }
+	  else {
+	    code.replace(line,1,"H");
+	  }
+	  e=new SyGpioEvent(SyGpioEvent::TypeGpi,addr,port,srcnum,line,state,
+			    false);
+	  emit gpioReceived(e);
+	  emit gpiReceived(e->sourceNumber(),e->line(),e->state(),e->isPulse());
+	  gpio_routing->
+	    setGpi(e->sourceNumber(),e->line(),e->state(),e->isPulse());
+	  delete e;
+	}
+	gpio_gpi_codes[srcnum]=code.toLower();
+	eb=
+	  new SyGpioBundleEvent(SyGpioBundleEvent::TypeGpi,addr,port,srcnum,code);
+	emit gpioReceived(eb);
+	delete eb;
       }
     }
   }
