@@ -29,6 +29,7 @@
 
 #include <QDateTime>
 #include <QHostAddress>
+#include <QMap>
 #include <QString>
 #include <QTimer>
 
@@ -36,6 +37,31 @@
 #include <sy/syethmonitor.h>
 #include <sy/syrouting.h>
 
+class SyGpioBundleEvent
+{
+ public:
+  enum Type {TypeGpi=0,TypeGpo=1};
+  SyGpioBundleEvent(Type type,const QHostAddress &orig_addr,uint16_t orig_port,
+		    int srcnum,const QString &code);
+  Type type() const;
+  QHostAddress originAddress() const;
+  uint16_t originPort() const;
+  QHostAddress sourceAddress() const;
+  int sourceNumber() const;
+  QString code() const;
+  QString dump() const;
+
+ private:
+  Type event_type;
+  QHostAddress event_origin_address;
+  uint16_t event_origin_port;
+  int event_source_number;
+  QString event_code;
+};
+
+
+
+
 class SyGpioEvent
 {
  public:
@@ -45,6 +71,7 @@ class SyGpioEvent
   Type type() const;
   QHostAddress originAddress() const;
   uint16_t originPort() const;
+  QHostAddress sourceAddress() const;
   int sourceNumber() const;
   int line() const;
   bool state() const;
@@ -73,10 +100,13 @@ class SyGpioServer : public QObject
   ~SyGpioServer();
 
  public slots:
-  void sendGpi(int gpi,int line,bool state,bool pulse);
-  void sendGpo(int gpo,int line,bool state,bool pulse);
+  void sendGpi(int srcnum,int line,bool state,bool pulse);
+  void sendGpi(int srcnum,const QString &code);
+  void sendGpo(int srcnum,int line,bool state,bool pulse);
+  void sendGpo(int srcnum,const QString &code);
 
  signals:
+  void gpioReceived(SyGpioBundleEvent *e);
   void gpioReceived(SyGpioEvent *e);
   void gpiReceived(int gpi,int line,bool state,bool pulse);
   void gpoReceived(int gpo,int line,bool state,bool pulse);
@@ -94,6 +124,7 @@ class SyGpioServer : public QObject
   uint32_t gpio_serial;
   std::map<uint32_t,uint32_t> gpio_src_addr_serials;
   SyEthMonitor *gpio_eth_monitor;
+  QMap<int,QString> gpio_gpi_codes;
 };
 
 
